
__author__  = 'SvaRoX'
__version__ = '0.2'


import b3
import b3.events
import b3.plugin
import time, thread, threading

#--------------------------------------------------------------------------------------------------
class TeamleadsPlugin(b3.plugin.Plugin):
    _adminPlugin = None
    _minLevel = 100
    _stfu = 0
    _redTeamPoints = 0
    _blueTeamPoints = 0

    _fragsLeft = 0
    _fragsLeftRedDisp = 0
    _fragsLeftBlueDisp = 0
    _fragLimit = 0
    _timeLeft = 0
    _timeLimit = 0
    _threadTime = None

    def onLoadConfig(self):
        self._adminPlugin = self.console.getPlugin('admin')
        if not self._adminPlugin:
            self.error('Could not find admin plugin')
            return False
        
        # Loads settings
        try:
            self._minLevel = self.config.getint('settings', 'minlevel')
        except:
            pass
        self.debug('Minimum Level to use commands = %d' % self._minLevel)

        try:
            self._fragsLeft = self.config.getint('settings', 'fragsleft')
            self.debug('FragsLeft : %d', self._fragsLeft)
        except:
            self.debug('Fraglefts value not found, X frags left will not be displayed')

        try:
            self._timeLeft = self.config.getint('settings', 'timeleft')
        except:
            pass
        self.debug('Timeleft : %d', self._timeLeft)

        self._adminPlugin.registerCommand(self, 'tlstfu', self._minLevel, self.cmd_stfu)

        # Loads CVars
        self._fragLimit = self.console.getCvar('fraglimit').getInt()
        self.debug('Fraglimit : %d', self._fragLimit)
        # Loads time limit according to the g_gametype (timelimit or g_RoundTime)
        if self.console.getCvar('g_gametype') in [0, 3, 7]:
            self._timeLimit = self.console.getCvar('timelimit').getInt()
        else:
            self._timeLimit = self.console.getCvar('g_roundtime').getInt()
        self._timeLimit = self.console.getCvar('timelimit').getInt()
        self.debug('Timelimit : %d', self._timeLimit)

    def onStartup(self):
        self.registerEvent(b3.events.EVT_GAME_ROUND_START)
        self.registerEvent(b3.events.EVT_GAME_EXIT)
        # '-> See poweradmin 604
        self.registerEvent(b3.events.EVT_CLIENT_KILL)
        self.registerEvent(b3.events.EVT_CLIENT_KILL_TEAM)
        self.registerEvent(b3.events.EVT_CLIENT_SUICIDE)
    
    def onEvent(self, event):
        diff = self._redTeamPoints - self._blueTeamPoints
        if event.type == b3.events.EVT_GAME_EXIT:
            self.resetScores()
            try:
                self._threadTime.cancel()
                self.debug('Thread cancelled')
            except:
                self.debug('Thread not cancelled')
                # pass
        elif event.type == b3.events.EVT_GAME_ROUND_START:
            self.resetScores()
            if self._timeLeft > 0:
                self.resetTimeLeft()
        elif event.type == b3.events.EVT_CLIENT_KILL:
            self.someoneKilled(event.client)
            self.processScore(diff)
        elif (event.type == b3.events.EVT_CLIENT_SUICIDE) or (event.type == b3.events.EVT_CLIENT_KILL_TEAM):
            self.someoneDead(event.client)
            self.processScore(diff)
    
    def cmd_stfu(self, data, client, cmd=None):
        """\
        Enable/disable silent mode (no more bigtexts)
        """
        msg = ['off', 'on']
        self._stfu = (self._stfu + 1) % 2
        cmd.sayLoudOrPM(client, '^7TeamLeads plugin : silent mode %s' % msg[self._stfu])

    def resetScores(self):
        self._redTeamPoints = 0
        self._blueTeamPoints = 0
        self._fragsLeftRedDisp = 0
        self._fragsLeftBlueDisp = 0
    
    def resetTimeLeft(self):
        # See threading poweradmin 638
        # duration = self._timeLimit * 60 - self._timeLeft
        duration = self._timeLimit * 60 - self._timeLeft - self.console.game.roundTime()
        # if negative duration : warmup time
        if duration < 0:
            duration = self._timeLimit * 60 - self._timeLeft
        try:
            self._threadTime.cancel()
        except:
            pass
            # self.debug('Timer doesn\'t exists, cancel cancelled')
        # Because of "- roundTime()", duration sometimes gets negative before warmup
        if duration > 0:
            self._threadTime = threading.Timer(duration, self.processTime)
            self.debug('Starting time left timer : %d seconds' % duration)
            self._threadTime.start()

    def someoneKilled(self, client):
        col = ''
        if client.team == b3.TEAM_RED:
            self._redTeamPoints = self._redTeamPoints + 1
        else:
            self._blueTeamPoints = self._blueTeamPoints + 1

    def someoneDead(self, client):
        col = ''
        if client.team == b3.TEAM_RED:
            self._redTeamPoints = self._redTeamPoints - 1
        else:
            self._blueTeamPoints = self._blueTeamPoints - 1

    def processScore(self, diff):
        self.debug('Processing team scores')
        msg = ''
        newdiff = self._redTeamPoints - self._blueTeamPoints
        if diff == 0:
            if newdiff > 0:
                msg = '^1Red team ^7has taken the lead !'
            else:
                msg = '^4Blue team ^7has taken the lead !'
        else:
            if newdiff == 0:
                msg = 'Teams are tied'
        
        if (msg != '') and (self._stfu != 1):
            # self.debug('Displays bigtext %s, diff = %d, redteampoints = %d, blueteampoints = %d' % (msg, diff, self._redTeamPoints, self._blueTeamPoints))
            self.console.write('bigtext "%s"' % msg)
            pass

        if (self._fragsLeft > 0) and (self._stfu != 1):
            if (self._fragsLeftRedDisp == 0) and (self._fragLimit - self._fragsLeft == self._redTeamPoints):
                self._fragsLeftRedDisp = 1
                self.debug('Displays bigtext red left : %d' % self._fragsLeft)
                self.console.write('bigtext "^1%d frags left"' % self._fragsLeft)
            elif (self._fragsLeftBlueDisp == 0) and (self._fragLimit - self._fragsLeft == self._blueTeamPoints):
                self._fragsLeftBlueDisp = 1
                self.debug('Displays bigtext blue left : %d' % self._fragsLeft)
                self.console.write('bigtext "^4%d frags left"' % self._fragsLeft)

    def processTime(self):
        intTime = self._timeLeft
        unit = 'seconds'
        if self._timeLeft >=60:
            intTime = int(self._timeLeft / 60)
            if intTime == 1:
                unit = 'minute'
            else:
                unit = 'minutes'
        if self._stfu != 1:
            self.console.write('bigtext "%d %s left"' % (intTime, unit))
